"""
Django settings for MRIOutils project.

Generated by 'django-admin startproject' using Django 3.1.7.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

################################################################################
####################### ATTENTION!!!!!!!!!!!!!!!!! #############################
####################### Ne pas editer ces settings #############################
######## si besoin d'en modifier un le copier dans settings_local.py ########### 
################################################################################

from pathlib import Path
import passlib.hash

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

STATIC_ROOT = '{}/static/'.format(BASE_DIR)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

LOGIN_URL = '/'

SESSION_EXPIRE_SECONDS = 5*60  # 5 min
SESSION_EXPIRE_AFTER_LAST_ACTIVITY = True

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'crispy_forms',
    'mri_links',
    'mri_users.apps.MriUsersConfig',
    'mri_db_logger.apps.MriDbLoggerConfig',
    'ckeditor',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django_session_timeout.middleware.SessionTimeoutMiddleware',
]

MIDDLEWARE_CLASSES = (
   'django.middleware.locale.LocaleMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
   "django.core.context_processors.i18n",
)

ROOT_URLCONF = 'MRIOutils.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'MRIOutils.wsgi.application'


# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        'OPTIONS': {
            'min_length': 8,
        },
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
    {
        'NAME': 'mri_users.custom_password_validation.ContainSpecialChar',
        'OPTIONS': {
            'special_char_count': 1,
            'special_char_list': ['@', '-', '_', '*', '$', '+', '=', '#', '!', '&'],
        },
    },
    {
        'NAME': 'mri_users.custom_password_validation.ContainLetterChar',
        'OPTIONS': {
            'letter_count': 1,
        },
    },
    {
        'NAME': 'mri_users.custom_password_validation.ContainDigit',
        'OPTIONS': {
            'digit_count': 1,
        },
    },
]

AUTH_USER_MODEL = 'mri_users.User'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'WARNING',
    },
}

# LOGGING = {
#     "version": 1,
#     "disable_existing_loggers": False,
#     "handlers": {"console": {"class": "logging.StreamHandler"}},
#     "loggers": {"django_auth_ldap": {"level": "DEBUG", "handlers": ["console"]}},
# }

AUTH_LDAP_USER_ATTR_MAP = {"first_name": "givenName", "last_name": "sn",
                           "username": "uid", "email": "mail",
                           "account_type": "accountType"}

# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = 'fr-FR'

TIME_ZONE = 'Europe/Paris'

USE_I18N = True

USE_L10N = True

USE_TZ = True

gettext = lambda x: x

LANGUAGES = (
   ('fr', gettext('French')),
   ('en', gettext('English')),
)

CRISPY_TEMPLATE_PACK="bootstrap4"

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = '/static/'

'''
Liste des algos possible:
ldap_bcrypt
ldap_bsdi_crypt
ldap_des_crypt
ldap_hex_md5
ldap_hex_sha1
ldap_md5
ldap_md5_crypt
ldap_pbkdf2_sha1
ldap_pbkdf2_sha256
ldap_pbkdf2_sha512
ldap_plaintext
ldap_salted_md5
ldap_salted_sha1
ldap_salted_sha256
ldap_salted_sha512
ldap_sha1
ldap_sha1_crypt
ldap_sha256_crypt
ldap_sha512_crypt
Attention ces algo servent au hash des mots de passe initialement, au changement c'est le serveur qui prend son algo par defaut.
'''

ENCRYPT_ALGO = passlib.hash.ldap_salted_sha1

RESET_LINK_PWD_HOUR_VALIDITY = 1

NEWUSER_VALIDATE_EMAIL_HOUR_VALIDITY = 48

DBLOG_DELETE_AFTER_DAYS = 365

MAX_USERNAME_LENGTH = 20

CKEDITOR_CONFIGS = {
    'default': {
        'toolbar': 'Custom',
        'toolbar_Custom': [
            {'name': 'basicstyles', 'items': ['Bold', 'Italic', 'Underline',
                    'Strike', '-', 'RemoveFormat']},
            {'name': 'paragraph', 'items': ['NumberedList', 'BulletedList', '-',
                    'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock']},
            {'name': 'clipboard', 'items': ['Undo', 'Redo']},
            {'name': 'styles', 'items': ['Styles']},
            {'name': 'colors', 'items': ['TextColor']},
            {'name': 'links', 'items': ['Link']},
            {'name': 'insert', 'items': ['Table']},
            {'name': 'tools', 'items': ['Maximize', 'ShowBlocks']},
            {'name': 'document', 'items': ['Source']},
        ]
    },
}

from .settings_local import *
