'''
Created on 9 mars 2021

@author: laborie
'''
import ldap
import re
import ldif
import ldap.modlist as modlist
from .models import User,UserTrash
from django.core import mail
from django.template.loader import render_to_string
from django.utils.html import strip_tags
from django.conf import settings
from django_auth_ldap.backend import LDAPBackend
from io import StringIO
from smtplib import SMTPException


def mail_with_template(template, to_mail, **kwargs):
    """
    Envoi la demande de confirmation mail
    """
    try:
        subject = "MRI"
        html_message = render_to_string(template, kwargs)
        title_search = re.findall(r'<title>(.+?)</title>', html_message)
        if len(title_search):
            subject = title_search[0]
        html_message_body = html_message.split('</head>')[1]
        plain_message = '\n'.join([s for s in strip_tags(html_message_body).splitlines() if not s.isspace()])
        from_email = settings.EMAIL_FROM
        to = to_mail
        if not isinstance(to, list):
            to = [to]
        mail.send_mail(subject, plain_message, from_email, to, html_message=html_message)
    except SMTPException:
        return False
    return True

# LDAP Stuff

def sync_ldap():
    '''
    Synchro du LDAP Complet
    '''
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s()
    users = l.search_ext_s(settings.AUTH_LDAP_BASE, ldap.SCOPE_SUBTREE, "(objectClass=posixAccount)", attrlist=["uid"])
    user_list = [user[1]['uid'][0].decode() for user in users]
    User.objects.all().exclude(username__in=user_list).delete()
    for user in user_list:
        sync_one_uid(user)

def sync_one_uid(uid):
    '''
    Synchro des infos d'un uid ldap
    '''
    LDAPBackend().populate_user(uid)

def getnextuidandincrement():
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    dn = "cn=NextFreeUnixId,dc=mri.cnrs,dc=fr"
    newuid = l.search_s(dn, ldap.SCOPE_BASE, filterstr=None)[0][1]['uidNumber'][0].decode()
    old = {'uidNumber': [newuid.encode('utf-8')]}
    attrs = {'uidNumber': [str(int(newuid) + 1).encode('utf-8')]}
    ldif = modlist.modifyModlist(old, attrs)
    l.modify_s(dn,ldif)
    l.unbind_s()
    return newuid

def user_to_ldap(user):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    dn = "uid={},{}".format(user.username, settings.AUTH_LDAP_BASE)
    attrs = {}
    attrs['cn'] = [user.username.encode('utf-8')]
    attrs['givenName'] = [user.first_name.encode('utf-8')]
    attrs['sn'] = [user.last_name.encode('utf-8')]
    attrs['mail'] = [user.email.encode('utf-8')]
    attrs['accountType'] = [b"user"]
    attrs['uidNumber'] = [getnextuidandincrement().encode('utf-8')]
    attrs['gidNumber'] = ['513'.encode('utf-8')]
    attrs['homeDirectory'] = ['/home/alluser'.encode('utf-8')]
    attrs['homePhone'] = [('/ftp2/{}'.format(user.username)).encode('utf-8')]
    attrs['userPassword'] = [user.password.encode('utf-8')]
    attrs['objectclass'] = [b"top", b"inetOrgPerson", b"posixAccount", b"mri-user"]
    ldif = modlist.addModlist(attrs)
    try:
        l.add_s(dn,ldif)
    except ldap.LDAPError as e:
        print(e)
    else:
        LDAPBackend().populate_user(user.username)
        user.delete()
    finally:
        l.unbind_s()

def change_ldap_user_data(user, old_attr, new_attr):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    dn = "uid={},{}".format(user.username, settings.AUTH_LDAP_BASE)
    ldif = modlist.modifyModlist(old_attr, new_attr)
    try:
        l.modify_s(dn,ldif)
    except ldap.LDAPError as e:
        print(e)
    finally:
        l.unbind_s()
        LDAPBackend().populate_user(user.username)

def change_user_passwd(user, password, old=None, changed_with_link=False):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    dn = "uid={},{}".format(user.username, settings.AUTH_LDAP_BASE)
    try:
        l.passwd_s(dn, old, password)
    except ldap.LDAPError as e:
        print(e)
    finally:
        if changed_with_link:
            user.locked = False
            user.save()
        l.unbind_s()

def verify_password(user, password):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    dn = "uid={},{}".format(user.username, settings.AUTH_LDAP_BASE)
    try:
        l.simple_bind_s(dn, password)
    except ldap.INVALID_CREDENTIALS:
        l.unbind_s()
        return False
    else:
        l.unbind_s()
        return True

def delete_ldap_user(user):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    dn = "uid={},{}".format(user.username, settings.AUTH_LDAP_BASE)
    try:
        users = l.search_s(settings.AUTH_LDAP_BASE, ldap.SCOPE_SUBTREE,
                               filterstr="(uid={})".format(user.username), attrlist=["*"])
        user_ldif_file = StringIO()
        user_ldif = ldif.LDIFWriter(user_ldif_file)
        user_ldif.unparse(users[0][0], users[0][1])
        UserTrash(username=user.username,
                  ldif=user_ldif_file.getvalue()).save()
        l.delete(dn)
    except ldap.LDAPError as e:
        print(e)
    user.delete()

def restore_user_from_trash(user):
    l = ldap.initialize(settings.AUTH_LDAP_SERVER_URI)
    l.simple_bind_s(settings.AUTH_LDAP_BIND_DN, settings.AUTH_LDAP_BIND_PASSWORD)
    ldif_file = StringIO(user.ldif)
    parser = ldif.LDIFRecordList(ldif_file)
    parser.parse()
    for dn, entry in parser.all_records:
        add_modlist = modlist.addModlist(entry)
        l.add_s(dn, add_modlist)
    sync_one_uid(user.username)
    user.delete()
