'''
Created on 17 mai 2021

@author: laborie
'''
from django.test import TestCase
from model_bakery import baker
from mri_users.models import User, NewUser

class UserTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        user1 = User.objects.create_user(username='testuser1',
                                         password='test1passw0rd')
        user1.save()

    def test_defaut_account_type(self):
        self.assertEqual(User.objects.get(pk=1).account_type, 'user')

    def test_account_type_admin(self):
        user1 = User.objects.get(pk=1)
        user1.account_type = 'administrator'
        user1.save()
        self.assertEqual(User.objects.get(pk=1).account_type, 'administrator')


class NewUserTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        baker.make(NewUser, _quantity=10, _fill_optional=True)

    def test_username_max_length(self):
        new_user = NewUser.objects.get(id=1)
        max_length = new_user._meta.get_field('username').max_length
        self.assertEqual(max_length, 150)

    def test_first_name_max_length(self):
        new_user = NewUser.objects.get(id=1)
        max_length = new_user._meta.get_field('first_name').max_length
        self.assertEqual(max_length, 150)
 
    def test_last_name_max_length(self):
        new_user = NewUser.objects.get(id=1)
        max_length = new_user._meta.get_field('last_name').max_length
        self.assertEqual(max_length, 150)

    def test_password_max_length(self):
        new_user = NewUser.objects.get(id=1)
        max_length = new_user._meta.get_field('password').max_length
        self.assertEqual(max_length, 200)

    def test_default_email_confirmed_false(self):
        new_user = NewUser.objects.get(id=1)
        self.assertFalse(new_user.email_confirmed)

    def test_generate_username(self):
        users = NewUser.objects.all()
        for user in users:
            self.assertTrue(len(user.username) > 2)
            self.assertTrue(user.username.isalpha(),
                            msg='{} is not alpha'.format(user.username))

    def test_generate_username_caractere_speciaux(self):
        user = baker.make(NewUser, first_name='jean-noël', last_name='de La Bath')
        self.assertTrue(len(user.username) > 2)
        self.assertTrue(user.username.isalpha(),
                        msg='{} is not alpha'.format(user.username))

    def test_generate_username_cyrillic(self):
        user = baker.make(NewUser, first_name='Екатерина', last_name='Елизавета')
        self.assertTrue(len(user.username) > 2)
        self.assertTrue(user.username.isalpha(),
                        msg='{} is not alpha'.format(user.username))

    def test_generate_username_ideogramme(self):
        user = baker.make(NewUser, first_name='张伟', last_name='张杰')
        self.assertTrue(len(user.username) > 2)
        self.assertTrue(user.username.isalpha(),
                        msg='{} is not alpha'.format(user.username))

